@@ -24,10 +24,11 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
 import re, urllib
+from types import ListType, TupleType
 
 from imdb.Movie import Movie
 from imdb.Person import Person
-from imdb.utils import analyze_title, analyze_name, canonicalName
+from imdb.utils import analyze_title, analyze_name, canonicalName, re_episodes
 from imdb._exceptions import IMDbDataAccessError
 from imdb.parser.http import IMDbHTTPAccessSystem, imdbURL_search, \
                                 imdbURL_movie, imdbURL_person
@@ -55,7 +56,7 @@ def _unHtml(s):
     return subSGMLRefs(re_spacessub(' ', re_unhtmlsub('', s)).strip())
 
 
-_ltypes = (type([]), type(()))
+_inttype = type(0)
 
 def _getTagWith(s, cont):
     """Return the html tags in the 's' string containing the 'cont'
@@ -80,7 +81,7 @@ def _findBetween(s, begins, ends, beginindx=0):
     bi = s.find(begins, beginindx)
     if bi != -1:
         lbegins = len(begins)
-        if type(ends) in _ltypes:
+        if isinstance(ends, (ListType, TupleType)):
             eset = [s.find(end, bi+lbegins) for end in ends]
             eset[:] = [x for x in eset if x != -1]
             if not eset: ei = -1
@@ -184,6 +185,47 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             raise IMDbDataAccessError, 'unable to get movieID "%s"' % movieID
         title = _unHtml(title[0])
         d = analyze_title(title, canonical=1)
+        tv_series = _findBetween(cont, 'TV Series:</b>', '</a>')
+        if tv_series: mid = re_imdbID.findall(tv_series[0])
+        else: mid = None
+        if tv_series and mid:
+            s_title = _unHtml(tv_series[0])
+            s_data = analyze_title(s_title, canonical=1)
+            m = Movie(movieID=str(mid), data=s_data, accessSystem='mobile')
+            d['kind'] = 'episode'
+            d['episode of'] = m
+            d['movieID'] = mid[0]
+            ep_title = _findBetween(cont, '<h1><strong class="title">',
+                                            '</strong></h1>')
+            if ep_title:
+                ep_title = _findBetween(ep_title[0], '<small>', '</small>')
+            if ep_title:
+                s_data = analyze_title(ep_title[0], canonical=1)
+                del s_data['kind']
+                d.update(s_data)
+        air_date = _findBetween(cont, 'Original Air Date:</b>', '<br><br>')
+        if air_date:
+            air_date = air_date[0]
+            vi = air_date.find('(')
+            if vi != -1:
+                date = air_date[:vi].strip()
+                if date != '????':
+                    d['original air date'] = date
+                air_date = air_date[vi:]
+                season = _findBetween(air_date, 'Season', ',')
+                if season:
+                    season = season[0].strip()
+                    try: season = int(season)
+                    except: pass
+                    if season or type(season) is _inttype:
+                        d['season'] = season
+                episode = _findBetween(air_date, 'Episode', ')')
+                if episode:
+                    episode = episode[0].strip()
+                    try: episode = int(episode)
+                    except: pass
+                    if episode or type(season) is _inttype:
+                        d['episode'] = episode
         direct = _findBetween(cont, 'Directed by</b><br>', '<br> <br>')
         if direct:
             dirs = self._getPersons(direct[0], aonly=1)
@@ -200,7 +242,12 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             # Cover, the new style.
             short_title = d.get('title', u'')
             if short_title:
+                if d.get('kind') in ('tv series', 'tv mini series'):
+                    short_title = '&#34;%s&#34;' % short_title
                 cvurl = _getTagWith(cont, 'alt="%s"' % short_title)
+                if not cvurl:
+                    cvurl = _getTagWith(cont, 'alt="%s"' %
+                                        short_title.replace('#34', 'quot'))
                 cvurl[:] = [x for x in cvurl if x[0:4] == '<img']
                 if cvurl:
                     cvurl = _findBetween(cvurl[0], 'src="', '"')
@@ -264,9 +311,18 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         if mpaa: d['mpaa'] = _unHtml(mpaa[0])
         runtimes = _findBetween(cont, 'Runtime:</b>', '<br>')
         if runtimes:
-            rt = [x.strip().replace(' min', '')
-                    for x in runtimes[0].split('/')]
-            d['runtimes'] = rt
+            # XXX: number of episodes?
+            runtimes = runtimes[0]
+            episodes = re_episodes.findall(runtimes)
+            if episodes:
+                runtimes = re_episodes.sub('', runtimes)
+                episodes = episodes[0]
+                try: episodes = int(episodes)
+                except: pass
+                d['number of episodes'] = episodes
+            runtimes = [x.strip().replace(' min', '')
+                    for x in runtimes.split('/')]
+            d['runtimes'] = runtimes
         country = _findBetween(cont, 'href="/Sections/Countries/', ['"', '/'])
         if country: d['countries'] = country
         lang = _findBetween(cont, 'href="/Sections/Languages/', ['"', '/'])
@@ -362,7 +418,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             hs[:] = _findBetween(hs[0], 'src="', '"')
             if hs: r['headshot'] = hs[0]
         workkind = _findBetween(s, '<b><a name=', '</a> - filmography')
-        
+
         ws = []
         for w in workkind:
             if w and w[0] == '"': w = w[1:]
@@ -375,6 +431,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             se = se.strip()
             sn = w[eti+1:].strip().lower()
             if se and sn: ws.append((se, sn))
+        # XXX: I think "guest appearances" are gone.
         if s.find('<a href="#guest-appearances"') != -1:
             ws.append(('guest-appearances', 'notable tv guest appearances'))
         if s.find('<a href="#archive">') != -1:
@@ -413,8 +470,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                     if fae != -1:
                         m = m[:fai] + m[fai+fae+4:]
                 tvi = m.find('<small>TV Series</small>')
-                if tvi != -1 and d['title'][0] != '"':
-                    d['title'] = '"%s"' % d['title']
+                if tvi != -1:
                     m = m[:tvi] + m[tvi+24:]
                 m = m.strip()
                 for x in xrange(2):
@@ -495,4 +551,3 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             d[sect] = data
         return {'data': d}
 
-
